local _M = {}

local sp=this.getSharedPreferences("Editor",0)
local editor = sp.edit()

function set(v)
  editor.putBoolean("showError",v).apply()
end

function get()
  return sp.getBoolean("showError",false)
end

_M.set = set
_M.get = get

local Snackbar = luajava.bindClass "com.google.android.material.snackbar.Snackbar"

local function snack(content)
  Snackbar.make(mContent,tostring(content),Snackbar.LENGTH_SHORT).setAnchorView(ps_bar).show()
end

local function copy(text)
  local s = this.getSystemService(activity.CLIPBOARD_SERVICE)
  s.setText(tostring(text))
end

local function read(tl_path)
  local file = io.open(tl_path, "r")
  if file then
    local content = file:read("*a")
    file:close()
    return content
  end
  return nil
end

_M.checkCode = function()
  local src = mLuaEditor.getText().toString()
  local path = mSubTitle.getText()

  if ends(path,".aly") then
    src = "return " .. src

    local _, data = loadstring(src)
    if data then
      local _, _, line, data = data:find(".(%d+).(.+)")
      mLuaEditor.gotoLine(tonumber(line))
      MaterialAlertDialogBuilder(this)
      .setTitle("Aly检查")
      .setMessage(line .. ":" .. data)
      .setPositiveButton(android.R.string.ok,nil)
      .setNegativeButton(android.R.string.copy,function()
        copy(line .. ":" .. data)
      end)
      .show()
      return true
     else
      snack("没有语法错误")
    end

   elseif ends(path,".lua") then

    local _, data = loadstring(src)
    if data then
      local _, _, line, data = data:find(".(%d+).(.+)")
      mLuaEditor.gotoLine(tonumber(line))
      MaterialAlertDialogBuilder(this)
      .setTitle("Lua检查")
      .setMessage(line .. ":" .. data)
      .setPositiveButton(android.R.string.ok,nil)
      .setNegativeButton(android.R.string.copy,function()
        copy(line .. ":" .. data)
      end)
      .show()
      return true
     else
      snack("没有语法错误")
    end

   elseif ends(path,".tl")

    local code = src
    local result = tl.process_string(code,false)
    code=""

    if result.syntax_errors and (#result.syntax_errors ~= 0) then
      for k,v in pairs(result.syntax_errors)
        code = code .. v.y .. ":" .. v.x .. ":  " ..v.msg .. "\n"
      end
    end
    if result.type_errors and (#result.type_errors ~= 0)
      for k,v in pairs(result.type_errors)
          code = code .. v.y .. ":" .. v.x .. ":  " ..v.msg .. "\n"
      end
    end
    if code ~= "" then
      MaterialAlertDialogBuilder(this)
      .setTitle("Teal检查")
      .setMessage(code)
      .setPositiveButton(android.R.string.ok,nil)
      .setNegativeButton(android.R.string.copy,function()
        copy(code)
      end)
      .show()
     else
      snack("没有错误")
    end

   else
    snack("不支持当前文件")
  end

end

_M.getError = function()
  local src = mLuaEditor.getText().toString()
  local path = mSubTitle.getText()

  if ends(path,".aly") then
    src = "return " .. src

    local _, data = loadstring(src)
    if data then
      local _, _, line, data = data:find(".(%d+).(.+)")
      return line .. ":" .. data
    end

   elseif ends(path,".lua") then

    local _, data = loadstring(src)
    if data then
      local _, _, line, data = data:find(".(%d+).(.+)")
      return line .. ":" .. data
    end

   elseif ends(path,".tl")

    local code = src
    local result = tl.process_string(code,false)
    code=""

    if result.syntax_errors and (#result.syntax_errors ~= 0) then
      for k,v in pairs(result.syntax_errors)
        code = code .. v.y .. ":" .. v.x .. ":  " ..v.msg .. "\n"
      end
    end
    if result.type_errors and (#result.type_errors ~= 0)
      for k,v in pairs(result.type_errors)
        if not v.msg:find[[no type information for required module]] then
          code = code .. v.y .. ":" .. v.x .. ":  " ..v.msg .. "\n"
        end
      end
    end
    if code ~= "" then
      return code
    end
  end
  return false

end


local textView,layout
_M.init=function(view)
  textView = view
  layout = textView.parent
  textView.onClick = function()
    _M.checkCode()
  end
  local ticker = Ticker()
  _M.ticker = ticker
  ticker.Period=200
  ticker.onTick=function()
    local error = _M.getError()
    if error then
      layout.visibility=0
      textView.text = error
     else
      layout.visibility=8
    end
  end
  ticker.start()
  ticker.setEnabled(_M.get())
end

_M.reset=function()
  local bool = _M.get()
  if _M.ticker then
    _M.ticker.setEnabled(bool)
  end
  if not bool then
    layout.visibility=8
  end
end

return _M